--TEST--
Complex Query Test
--DESCRIPTION--
Verifies the behavior of INSERT queries with and without the IDENTITY flag set.
--ENV--
PHPT_EXEC=true
--SKIPIF--
<?php require('skipif_versions_old.inc'); ?>
--FILE--
<?php

require_once('MsCommon.inc');

function complexQuery()
{
    $testName = "Statement - Complex Query";
    startTest($testName);

    setup();
    $conn1 = AE\connect();

    $tableName = 'TC33test';
    $columns = array(new AE\ColumnMeta('int', 'c1_int', "IDENTITY"),
                     new AE\ColumnMeta('tinyint', 'c2_tinyint'),
                     new AE\ColumnMeta('smallint', 'c3_smallint'),
                     new AE\ColumnMeta('bigint', 'c4_bigint'),
                     new AE\ColumnMeta('varchar(512)', 'c5_varchar'));
    AE\createTable($conn1, $tableName, $columns);

    // SET IDENTITY_INSERT ON/OFF only works at execute or run time and not at parse time
    // because a prepared statement runs in a separate context
    // https://technet.microsoft.com/en-us/library/ms188059(v=sql.110).aspx
    $query = "SET IDENTITY_INSERT [$tableName] ON;";
    $stmt = sqlsrv_query($conn1, $query);
    if (!$stmt) {
        die("Unexpected execution outcome for \'$query\'.");
    }
    
    // expect this to pass
    $inputs = array("c1_int" => -204401468, "c2_tinyint" => 168, "c3_smallint" => 4787, "c4_bigint" =>1583186637, "c5_varchar" => "<C~za.Oa._ߣ*<u_Coa a+OhobUa:zB_C@~Uz+//Z@o_:r,orzoZ*ߪ~ Ua>ZU/_ZhC+/.ob|,::*/>+/a.<:>O~*~z<.O,>,b@bhC*<<hb*oha+A/_@b/BB@~zZC@U_UhvU*a@:ZAAbU_b:orߪ_֪zoa <~zZaB.+A><:/Ur UOa:a|++.r~:/+|o++v_@BZ:AC./Ab<,>Ubb|ߣ:<<ba+,<_a._><|zz@>:a,Cr__.<C+UU_z b~o|, .,b/U>aBZ@ܣ: bvb>/@/b+r:Z>|uZAC:Ch *.ã_u|Ur.:aAUv@u>@<.<Z bZA֣o*,:");
    $stmt = insertTest($conn1, $tableName, true, $inputs);

    $query = "SET IDENTITY_INSERT [$tableName] OFF;";
    $stmt = sqlsrv_query($conn1, $query);
    if (!$stmt) {
        die("Unexpected execution outcome for \'$query\'.");
    }

    // expect this to fail
    $inputs = array("c1_int" => 1264768176, "c2_tinyint" => 111, "c3_smallint" => 23449, "c4_bigint" =>1421472052, "c5_varchar" => "uC@bUOv~,v,BZ*oh>zb_<@*OO_<u/or <bU~~ bܩ.uТ:|_Bbv@,<COv~:+,CZvhC/oUua<>/Ub,+AЩ:rB+~~+_<vO@ aCz@:r.~vh~r.b_Cr B:BbUvZ+|,CaAC,abb*UA hCuhOb |C.<C<.aBvu,Aa>ABU/O<Ouߣ~u+rb/:o  /_O:uzUvA_B/>UCr, aaãvZ@r*_::~/+.~abz*z<~rU~O+Z|A<_Bߩ ::.br/rh:U OA~Ar<v+hC/voU+O*B.Zbh/,>*U>abBbv/b| u.z~zU.UA*a*.> r ~Ca+r~@a/C*a,bb<o+v.u<B<BZu/_>*~");
    $stmt = insertTest($conn1, $tableName, false, $inputs);

    // expect this to pass
    $query = "SET IDENTITY_INSERT [$tableName] ON; SQL; SET IDENTITY_INSERT [$tableName] OFF;";
    if (AE\isColEncrypted()){
        // When AE is enabled, SQL types must be specified for sqlsrv_query
        $inputs = array("c1_int" => array(-411114769, null, null, SQLSRV_SQLTYPE_INT), 
                        "c2_tinyint" => array(198, null, null, SQLSRV_SQLTYPE_TINYINT),
                        "c3_smallint" => array(1378, null, null, SQLSRV_SQLTYPE_SMALLINT),
                        "c4_bigint" => array(140345831, null, null, SQLSRV_SQLTYPE_BIGINT), 
                        "c5_varchar" => array("@arêA*A>_hOv@|h~O<+*CbazaZ/:uazAh+u+r:| U*_v@@~Ch_*AABB,b.B+u*CAv,>CU<rz@r*ubBa@.Bvo~ o ou/>,,aO>C:Z><+r.bO.,uAr><ov:,+./||CU_Īh~<_/hb ĩuBu<@boBCA/: U*vu.Bo_br_>ܣBAvavCU  vu><UC*aUrhr+>||orУ<<|AohA_vu~:~h+Bu @Z+@h|@bU_/ |:zb>@Uߩ  o @B_BOBhCb~> rUzurbz/Uu.@__vBb/ruz*/*O", null, null, SQLSRV_SQLTYPE_VARCHAR(512)));
        $stmt = insertTest($conn1, $tableName, true, $inputs, $query);
    } else {
        $inputs = array("c1_int" => -411114769, "c2_tinyint" => 198, "c3_smallint" => 1378, "c4_bigint" => 140345831, "c5_varchar" => "@arêA*A>_hOv@|h~O<+*CbazaZ/:uazAh+u+r:| U*_v@@~Ch_*AABB,b.B+u*CAv,>CU<rz@r*ubBa@.Bvo~ o ou/>,,aO>C:Z><+r.bO.,uAr><ov:,+./||CU_Īh~<_/hb ĩuBu<@boBCA/: U*vu.Bo_br_>ܣBAvavCU  vu><UC*aUrhr+>||orУ<<|AohA_vu~:~h+Bu @Z+@h|@bU_/ |:zb>@Uߩ  o @B_BOBhCb~> rUzurbz/Uu.@__vBb/ruz*/*O");
        $stmt = insertTest($conn1, $tableName, true, $inputs, $query);
    }

    $stmt1 = selectFromTable($conn1, $tableName);
    $rowCount = rowCount($stmt1);
    sqlsrv_free_stmt($stmt1);

    if ($rowCount != 2) {
        die("Table $tableName has $rowCount rows instead of 2.");
    }

    dropTable($conn1, $tableName);

    sqlsrv_close($conn1);

    endTest($testName);
}

function insertTest($conn, $tableName, $expectedOutcome, $inputs, $query = null)
{
    $stmt = null;
    if (!AE\isColEncrypted()) {
        $insertSql = AE\getInsertSqlComplete($tableName, $inputs);
        if (! is_null($query)) {
            $sql = str_replace("SQL", $insertSql, $query);
        } else {
            $sql = $insertSql;
        }
        $stmt = sqlsrv_query($conn, $sql);
        $actualOutcome = ($stmt !== false);
    } else {
        // must bind parameters
        $insertSql = AE\getInsertSqlPlaceholders($tableName, $inputs);
        $params = array();
        foreach ($inputs as $key => $input) {
            array_push($params, $inputs[$key]);
        }
        if (! is_null($query)) {
            // this contains a batch of sql statements, 
            // with set identity_insert on or off 
            // thus, sqlsrv_query should be called
            $sql = str_replace("SQL", $insertSql, $query);
            $stmt = sqlsrv_query($conn, $sql, $params);
            $actualOutcome = ($stmt !== false);
        } else {
            // just a regular insert, so use sqlsrv_prepare
            $sql = $insertSql;
            $actualOutcome = true;
            $stmt = sqlsrv_prepare($conn, $sql, $params);
            if ($stmt) {
                $result = sqlsrv_execute($stmt);
                $actualOutcome = ($result !== false);
            }
        }
    }
    if ($actualOutcome != $expectedOutcome) {
        die("Unexpected execution outcome for \'$sql\'.");
    }
}

try {
    complexQuery();
} catch (Exception $e) {
    echo $e->getMessage();
}

?>
--EXPECT--
Test "Statement - Complex Query" completed successfully.
